import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import scipy.stats as ss

flnm = input('Data File Name (*.xlsx) = ')
xlsx = pd.ExcelFile(flnm)                   #   入力ファイル名の設定
data = pd.read_excel(xlsx)                  #   読み込み

VarLabels = ['Const'] + [v for v in data.columns[2:]]       #   独立変数名
print(VarLabels)
p = len(VarLabels)                          #   独立変数の数
print('p = ', p)
n = len(data.values)                        #   データ数
print('n = ', n)
YLabel = data.columns[1]                    #   従属変数名
Y = data[YLabel].values.reshape((n,1))      #   従属変数値
print(Y)
X = data.values[:, 2:]
print(X)
X = ss.zscore(X)                            #   独立変数の標準得点化
print(X)
X = np.hstack((np.array([1]*n).reshape((n,1)), X))      #   独立変数値
print(X)

b = np.linalg.inv(X.T @ X) @ X.T @ Y                    #   偏回帰係数の計算
#
#           95%信頼区間の計算
#
Yhat = X @ b
SSE = ((Yhat - Y) ** 2).sum()
MSE = SSE / (n - p)
s2_b = MSE * np.linalg.inv(X.T @ X)
s_b = np.diag(s2_b) ** 0.5
t975 = ss.t.ppf(0.975, n - p)
print()
for i in range(p):
    print(VarLabels[i] + ':')
    print('    b[{0:}] = {1:.3f}   95%CI = [{2:.3}, {3:.3}]'.format(i, b[i][0],
                                                                    b[i][0] - t975 * s_b[i],
                                                                    b[i][0] + t975 * s_b[i]))
#
#       多重決定係数と重相関係数の計算
#
Ymean = Y.mean()
print('\nY<{0:}>_mean = {1:}'.format(YLabel, Ymean))
SSReg = ((Yhat - Ymean) ** 2).sum()
SSTot = ((Y - Ymean) ** 2).sum()
R2 = SSReg / SSTot
print('\nR^2 = ', R2)
R = R2 ** 0.5
print('R = ', R)
